local screen = {}
local colors = {
  WATER = {4, 215, 255},
  SPLASH = {120, 233, 254},
  LEAF = {90, 216, 0},
  LEAF_BORDER = {72, 174, 0},
  PLAYER = {255, 247, 94}
}
local leaves = {}
local player = {}

function createLeaf(x,y,r)
  return {
    x = x,
    y = y,
    r = r,
    a = 2.0 * math.pi * math.random(),
    mult = 0.9 + 0.2 * math.random()
  }
end

function love.load()
  love.window.setTitle('Åkande')
  screen.w = love.graphics.getWidth()
  screen.h = love.graphics.getHeight()

  table.insert(leaves, createLeaf(150,150,30))
  table.insert(leaves, createLeaf(300,160,40))
  table.insert(leaves, createLeaf(260,270,50))
  table.insert(leaves, createLeaf(110,390,45))
  table.insert(leaves, createLeaf(400,300,66))

  player.x = 150
  player.y = 150
  player.z = 0
  player.vx = 0.0
  player.vy = 0.0
  player.vz = 0.0
  player.isJumping = false
  player.animated = false
  player.animTime = 1.0
end

function love.update(dt)
  for i = 1, #leaves do
    local leaf = leaves[i]
    leaf.a = leaf.a + 1.0 * math.pi * dt * leaf.mult
  end

  if not player.animated then
    player.animTime = player.animTime - dt

    if player.animTime <= 0.0 then
      player.animTime = 0.0
      player.animated = true
    end
  end

  if player.animated then
    local mx = 0.0
    local my = 0.0

    if love.keyboard.isDown('left') then mx = mx - 1.0 end
    if love.keyboard.isDown('right') then mx = mx + 1.0 end
    if love.keyboard.isDown('up') then my = my - 1.0 end
    if love.keyboard.isDown('down') then my = my + 1.0 end

    local d = math.sqrt(mx * mx + my * my)
    if d > 0.0 then
      mx = mx / d
      my = my / d
    end

    player.vx = player.vx + 1500.0 * mx * dt
    player.vy = player.vy + 1500.0 * my * dt

    local pd = math.sqrt(player.vx * player.vx + player.vy * player.vy)

    if pd > 350.0 then
      player.vx = player.vx / pd * 350.0
      player.vy = player.vy / pd * 350.0
    end

    player.x = player.x + player.vx * dt
    player.y = player.y + player.vy * dt
    player.z = player.z + player.vz * dt

    if player.z > 0.0 then
      player.vz = player.vz - 1000.0 * dt
    end

    if player.z <= 0.0 then
      player.z = 0.0
      player.vz = 0.0
      player.isJumping = false
    end

    if love.keyboard.isDown('z') and player.isJumping == false then
      player.isJumping = true
      player.vz = 500.0
    end

    if player.z < 0.01 then
      local died = true
      for i = 1, #leaves do
        local leaf = leaves[i]
        local dx = leaf.x - player.x
        local dy = leaf.y - player.y
        local d = math.sqrt(dx * dx + dy * dy)

        if d < leaf.r then
          died = false
          break
        end
      end
      if died then
        player.animated = false
        player.animTime = 1.0
        player.x = 150
        player.y = 150
        player.vx = 0
        player.vy = 0
        player.vz = 0

      end
    end
  end
end

function love.draw()
  love.graphics.clear()
  love.graphics.setColor(colors.WATER)
  love.graphics.rectangle("fill", 0, 0, screen.w, screen.h)
  love.graphics.setColor(colors.SPLASH)

  for i = 1, #leaves do
    local leaf = leaves[i]
    local radius = leaf.r + 3.0 * math.sin(leaf.a)

    love.graphics.setColor(colors.LEAF_BORDER)
    love.graphics.circle("fill", leaf.x, leaf.y, radius + 5, 32)
    love.graphics.setColor(colors.LEAF)
    love.graphics.circle("fill", leaf.x, leaf.y, radius, 32)
  end

  local displayPlayer = true

  if not player.isAnimated then
    displayPlayer = (player.animTime % 0.2) < 0.1
  end

  if displayPlayer then
    love.graphics.setColor(0, 0, 0, 50)
    love.graphics.circle("fill", player.x, player.y, 15 * (1 - (player.z / 400)), 32)
    love.graphics.setColor(colors.PLAYER)
    love.graphics.circle("fill", player.x, player.y - player.z, 15, 32)
  end
end
